/* Yvonne 1.0
 * by Rafal BOBER Ciepiela
 * contact: bob_er@users.sourceforge.net
 * 2006
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __YVONNE_COLORPROVIDER__
#define __YVONNE_COLORPROVIDER__

#include <exception>
#include <fstream>
#include <string>

////////////////////////////////////////////////////////

class CCPException:public std::exception
{
public:
	enum CPReason {
		CP_CANNOT_INITIALIZE=1,
		CP_CANNOT_OPEN_FILE=2,
		CP_CANNOT_READ_FILE=3,
		CP_OUT_OF_RANGE=4
	};

	CCPException(CPReason _eReason) throw():m_eReason(_eReason) {};
	CPReason getReason(void) { return m_eReason; }
	char* what(void)
	{
		switch(m_eReason)
		{
		case CP_CANNOT_INITIALIZE:
			return "Cannot initialize.";
		case CP_CANNOT_OPEN_FILE:
			return "Cannot open palette file.";
		case CP_CANNOT_READ_FILE:
			return "Cannot read palette file.";
		case CP_OUT_OF_RANGE:
			return "Color index is out of range.";
		}
		return "Unknown exception.";
	}

protected:
	CPReason m_eReason;

};

////////////////////////////////////////////////////////

class CColorProvider
{
public:
	CColorProvider(void);
	CColorProvider(std::string _sFile);
	~CColorProvider(void);
	int getColor(unsigned int _nIndex);
	bool open(std::string _sFile);

private:
	static const int m_vDefaultPalette[256];
	int* m_vPalette;
	std::string m_sFileName;

	void setDefault(void);

};

#endif /* __YVONNE_COLORPROVIDER__ */

////////////////////////////////////////////////////////
