/* Yvonne 1.0
 * by Rafal BOBER Ciepiela
 * contact: bob_er@users.sourceforge.net
 * 2006
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "gr8.h"

using namespace std;

////////////////////////////////////////////////////////

CGr8::CGr8(CColorProvider* _pColors):CImage(_pColors)
{
	m_cPicture=NULL;
	return;
}

////////////////////////////////////////////////////////

CGr8::~CGr8(void)
{
	if(m_cPicture!=NULL)
		delete m_cPicture;
	return;
}

////////////////////////////////////////////////////////

bool CGr8::open(char* _szPath)
{
	if(_szPath==NULL)
		throw CImageException(CImageException::IE_BAD_PARAMS);
	m_sFileName=_szPath;
	m_fInFile.open(_szPath, ios::in);
	if(m_fInFile.is_open()==false)
		throw CImageException(CImageException::IE_CANNOT_OPEN_FILE);
	m_cPicture=new unsigned char[IMAGE_SIZE_FULLSCREEN];
	m_fInFile.read(reinterpret_cast<char *>(m_cPicture), IMAGE_SIZE_FULLSCREEN);
	if(m_fInFile.eof()==true)
		throw CImageException(CImageException::IE_CANNOT_READ_FILE);
	m_fInFile.close();
	m_nWidth=320;
	m_nHeight=192;
	return true;
}

////////////////////////////////////////////////////////

unsigned int CGr8::getPixel(unsigned int _nX, unsigned int _nY)
{
	unsigned int nSeek;
	unsigned int nResult;

	if((_nX>=m_nWidth) || (_nY>=m_nHeight))
		throw CImageException(CImageException::IE_BAD_PARAMS);
	nSeek=(40*_nY)+(_nX>>3);
	nResult=m_cPicture[nSeek];
	switch(_nX&0x07)
	{
	case 0:
		nResult=((nResult&0x80)?0x0f:0x00);
		break;
	case 1:
		nResult=((nResult&0x40)?0x0f:0x00);
		break;
	case 2:
		nResult=((nResult&0x20)?0x0f:0x00);
		break;
	case 3:
		nResult=((nResult&0x10)?0x0f:0x00);
		break;
	case 4:
		nResult=((nResult&0x08)?0x0f:0x00);
		break;
	case 5:
		nResult=((nResult&0x04)?0x0f:0x00);
		break;
	case 6:
		nResult=((nResult&0x02)?0x0f:0x00);
		break;
	case 7:
		nResult=((nResult&0x01)?0x0f:0x00);
		break;
	default:
		throw CImageException(CImageException::IE_BAD_PARAMS);
	}
	nResult=m_pColorProvider->getColor(nResult);
	return nResult;
}

////////////////////////////////////////////////////////

const char* CGr8::getFileName(void)
{
	return m_sFileName.c_str();
}

////////////////////////////////////////////////////////

const char* CGr8::getFileSize(void)
{
	return "320x192; 2 colours";
}

////////////////////////////////////////////////////////

const char* CGr8::getFileProperties(void)
{
	return "No properties supported.";
}

////////////////////////////////////////////////////////
