/* Yvonne 1.0
 * by Rafal BOBER Ciepiela
 * contact: bob_er@users.sourceforge.net
 * 2006
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "hpm.h"

#include <iostream>

using namespace std;

////////////////////////////////////////////////////////

CHpm::CHpm(CColorProvider* _pColors):CImage(_pColors)
{
	m_cPicture=NULL;
	return;
}

////////////////////////////////////////////////////////

CHpm::~CHpm(void)
{
	if(m_cPicture!=NULL)
		delete m_cPicture;
	return;
}

////////////////////////////////////////////////////////

bool CHpm::open(char* _szPath)
{
	int nCount;
	int nMaxSize;

	if(_szPath==NULL)
		throw CImageException(CImageException::IE_BAD_PARAMS);
	m_sFileName=_szPath;
	m_fInFile.open(_szPath, ios::in);
	if(m_fInFile.is_open()==false)
		throw CImageException(CImageException::IE_CANNOT_OPEN_FILE);
	m_cPicture=new unsigned char[IMAGE_SIZE_FULLSCREEN+40];
	nCount=0;
	nMaxSize=(40*192)+1;
	while(nCount<(40*192))
	{
		int a;

		a=m_fInFile.get();
		if(a!=0)
		{
			if((nCount+a)>nMaxSize)
				throw CImageException(CImageException::IE_CANNOT_READ_FILE);
			m_fInFile.read(reinterpret_cast<char *>(m_cPicture+nCount), a);
			nCount+=a;
		}
		else
		{
			int nWhat;
			int nCount2;

			nWhat=m_fInFile.get();
			nCount2=m_fInFile.get();
			if((nCount+nCount2)>nMaxSize)
				throw CImageException(CImageException::IE_CANNOT_READ_FILE);
			for(int cnt=0;cnt<nCount2;cnt++)
				m_cPicture[nCount+cnt]=nWhat;
			nCount+=nCount2;
		}
		if(m_fInFile.eof()==true)
			throw CImageException(CImageException::IE_CANNOT_READ_FILE);
	}
	if(m_fInFile.eof()==true)
		throw CImageException(CImageException::IE_CANNOT_READ_FILE);
	m_fInFile.close();
	setPalette();
	m_nWidth=320;
	m_nHeight=192;
	return true;
}

////////////////////////////////////////////////////////

unsigned int CHpm::getPixel(unsigned int _nX, unsigned int _nY)
{
	unsigned int nX;
	unsigned int nSeek;
	unsigned int nResult;

	if((_nX>=m_nWidth) || (_nY>=m_nHeight))
		throw CImageException(CImageException::IE_BAD_PARAMS);
	nX=_nX>>1;
	nSeek=(40*_nY)+(nX>>2);
	nResult=m_cPicture[nSeek];
	switch(nX&3)
	{
	case 0:
		nResult=m_cPalette[(nResult&0xc0)>>6];
		break;
	case 1:
		nResult=m_cPalette[(nResult&0x30)>>4];
		break;
	case 2:
		nResult=m_cPalette[(nResult&0xc)>>2];
		break;
	case 3:
		nResult=m_cPalette[nResult&0x3];
		break;
	default:
		throw CImageException(CImageException::IE_BAD_PARAMS);
	}
	nResult=m_pColorProvider->getColor(nResult);
	return nResult;
}

////////////////////////////////////////////////////////

const char* CHpm::getFileName(void)
{
	return m_sFileName.c_str();
}

////////////////////////////////////////////////////////

const char* CHpm::getFileSize(void)
{
	return "160x192; 4 colours";
}

////////////////////////////////////////////////////////

const char* CHpm::getFileProperties(void)
{
	return "Author: Daniel Nowak (Grass/MEC).";
}

////////////////////////////////////////////////////////

void CHpm::setPalette(void)
{
	string sName;
	int nPos;

	nPos=m_sFileName.rfind("/");
	sName=m_sFileName.substr(nPos+1, m_sFileName.length());
	if(sName.compare("alien.hpm")==0)
	{
		m_cPalette[0]=0x00;
		m_cPalette[1]=0x34;
		m_cPalette[2]=0xc8;
		m_cPalette[3]=0x7c;
	}
	else if(sName.compare("czaszka.hpm")==0)
	{
		m_cPalette[0]=0xa4;
		m_cPalette[1]=0x51;
		m_cPalette[2]=0xb9;
		m_cPalette[3]=0x7c;
	}
	else if(sName.compare("dn.hpm")==0)
	{
		m_cPalette[0]=0x00;
		m_cPalette[1]=0x34;
		m_cPalette[2]=0xc8;
		m_cPalette[3]=0x7c;
	}
	else if(sName.compare("father.hpm")==0)
	{
		m_cPalette[0]=0x00;
		m_cPalette[1]=0x05;
		m_cPalette[2]=0x08;
		m_cPalette[3]=0x0c;
	}
	else if(sName.compare("fsily.hpm")==0)
	{
		m_cPalette[0]=0x00;
		m_cPalette[1]=0xe4;
		m_cPalette[2]=0xc8;
		m_cPalette[3]=0xbe;
	}
	else if(sName.compare("girl.hpm")==0)
	{
		m_cPalette[0]=0x06;
		m_cPalette[1]=0x04;
		m_cPalette[2]=0x00;
		m_cPalette[3]=0x0a;
	}
	else if(sName.compare("hpz.hpm")==0)
	{
		m_cPalette[0]=0x00;
		m_cPalette[1]=0x34;
		m_cPalette[2]=0xc8;
		m_cPalette[3]=0x7c;
	}
	else if(sName.compare("jordan.hpm")==0)
	{
		m_cPalette[0]=0x00;
		m_cPalette[1]=0x34;
		m_cPalette[2]=0x38;
		m_cPalette[3]=0x3c;
	}
	else if(sName.compare("kiss.hpm")==0)
	{
		m_cPalette[0]=0x0e;
		m_cPalette[1]=0x30;
		m_cPalette[2]=0xc7;
		m_cPalette[3]=0x7b;
	}
	else if(sName.compare("kopalny.hpm")==0)
	{
		m_cPalette[0]=0x00;
		m_cPalette[1]=0x35;
		m_cPalette[2]=0xc8;
		m_cPalette[3]=0x7c;
	}
	else if(sName.compare("raper.hpm")==0)
	{
		m_cPalette[0]=0x00;
		m_cPalette[1]=0x34;
		m_cPalette[2]=0xc8;
		m_cPalette[3]=0x7c;
	}
	else if(sName.compare("salem.hpm")==0)
	{
		m_cPalette[0]=0x00;
		m_cPalette[1]=0x34;
		m_cPalette[2]=0xc8;
		m_cPalette[3]=0x7c;
	}
	else if(sName.compare("star.hpm")==0)
	{
		m_cPalette[0]=0x00;
		m_cPalette[1]=0x64;
		m_cPalette[2]=0x58;
		m_cPalette[3]=0x7e;
	}
	else if(sName.compare("vivaldi.hpm")==0)
	{
		m_cPalette[0]=0x00;
		m_cPalette[1]=0x74;
		m_cPalette[2]=0x58;
		m_cPalette[3]=0x7e;
	}
	else if(sName.compare("zwiewka.hpm")==0)
	{
		m_cPalette[0]=0x00;
		m_cPalette[1]=0x34;
		m_cPalette[2]=0xc8;
		m_cPalette[3]=0x7c;
	}
	else
		throw CImageException(CImageException::IE_BAD_PARAMS);
	return;
}

////////////////////////////////////////////////////////
