/* Yvonne 1.0
 * by Rafal BOBER Ciepiela
 * contact: bob_er@users.sourceforge.net
 * 2006
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __YVONNE_IMAGE__
#define __YVONNE_IMAGE__

#include <exception>
#include <fstream>
#include <string>

#include "color_provider.h"

////////////////////////////////////////////////////////

#define RED(x) (x&0xff0000)>>16
#define GREEN(x) (x&0xff00)>>8
#define BLUE(x) (x&0xff)

////////////////////////////////////////////////////////

const unsigned int IMAGE_SIZE_FULLSCREEN=7680;	// default screen size
const unsigned int IMAGE_SIZE_INT=8000;					// int screen size
const unsigned int IMAGE_WIDTH_MAX=640;
const unsigned int IMAGE_HEIGHT_MAX=200;

////////////////////////////////////////////////////////

class CImageException : public std::exception
{
public:
	enum IEReason
	{
		IE_CANNOT_INITIALIZE=1,
		IE_BAD_PARAMS=2,
		IE_CANNOT_OPEN_FILE=3,
		IE_CANNOT_READ_FILE=4
	};

	CImageException(IEReason _eReason) throw():m_eReason(_eReason) {};
	IEReason getReason(void)
		{ return m_eReason; }
	char* what(void)
	{
		switch(m_eReason)
		{
		case IE_CANNOT_INITIALIZE:
			return "Cannot initialize.";
		case IE_BAD_PARAMS:
			return "Bad params.";
		case IE_CANNOT_OPEN_FILE:
			return "Cannot open file.";
		case IE_CANNOT_READ_FILE:
			return "Cannot read file.";
		}
		return "Unknown exception.";
	}

protected:
	IEReason m_eReason;
};

////////////////////////////////////////////////////////

class CImage
{
public:
	CImage(CColorProvider* _pColors);
	virtual ~CImage(void);

	virtual bool open(char* _szPath);
	virtual int getWidth(void);
	virtual int getHeight(void);
	virtual unsigned int getPixel(unsigned int _nX, unsigned int _nY);
	virtual const char* getFileName(void);
	virtual const char* getFileSize(void);
	virtual const char* getFileProperties(void);

protected:
	std::string m_sFileName;
	std::fstream m_fInFile;
	CColorProvider* m_pColorProvider;
	unsigned int m_nWidth;
	unsigned int m_nHeight;
};

#endif /* __YVONNE_IMAGE__ */

////////////////////////////////////////////////////////
