/* Yvonne 1.0
 * by Rafal BOBER Ciepiela
 * contact: bob_er@users.sourceforge.net
 * 2006
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "int.h"

using namespace std;

////////////////////////////////////////////////////////

CInt::CInt(CColorProvider* _pColors):CImage(_pColors)
{
	m_cPicture1=NULL;
	m_cPicture2=NULL;
	return;
}

////////////////////////////////////////////////////////

CInt::~CInt(void)
{
	if(m_cPicture1!=NULL)
		delete m_cPicture1;
	if(m_cPicture2!=NULL)
		delete m_cPicture2;
	return;
}

////////////////////////////////////////////////////////

bool CInt::open(char* _szPath)
{
	if(_szPath==NULL)
		throw CImageException(CImageException::IE_BAD_PARAMS);
	m_sFileName=_szPath;
	m_fInFile.open(_szPath, ios::in);
	if(m_fInFile.is_open()==false)
		throw CImageException(CImageException::IE_CANNOT_OPEN_FILE);
	m_cPicture1=new unsigned char[IMAGE_SIZE_INT];
	m_cPicture2=new unsigned char[IMAGE_SIZE_INT];
	m_fInFile.read(reinterpret_cast<char *>(m_cPicture1), 6);
	if(memcmp(m_cPicture1, "INT95a", 6)==0)
	{
		m_fInFile.seekg(18, ios::beg);
		m_fInFile.read(reinterpret_cast<char *>(m_cPicture1), IMAGE_SIZE_INT);
		m_fInFile.read(reinterpret_cast<char *>(m_cPicture2), IMAGE_SIZE_INT);
		m_cPalette[0]=0;
		m_cPalette[1]=0x28;
		m_cPalette[2]=0xc8;
		m_cPalette[3]=0x0f;
		
	}
	else
	{
		m_fInFile.seekg(0, ios::beg);
		m_fInFile.read(reinterpret_cast<char *>(m_cPicture1), IMAGE_SIZE_INT);
		m_fInFile.read(reinterpret_cast<char *>(m_cPicture2), IMAGE_SIZE_INT);
		m_fInFile.read(reinterpret_cast<char *>(m_cPalette), 4);
	}
	if(m_fInFile.eof()==true)
		throw CImageException(CImageException::IE_CANNOT_READ_FILE);
	m_fInFile.close();
	m_nWidth=320;
	m_nHeight=200;
	return true;
}

////////////////////////////////////////////////////////

unsigned int CInt::getPixel(unsigned int _nX, unsigned int _nY)
{
	unsigned int nX;
	unsigned int nSeek;
	unsigned int nColor1;
	unsigned int nColor2;
	unsigned int nResult;

	if((_nX>=m_nWidth) || (_nY>=m_nHeight))
		throw CImageException(CImageException::IE_BAD_PARAMS);
	nX=_nX>>1;
	nSeek=(40*_nY)+(nX>>2);
	nColor1=m_cPicture1[nSeek];
	nColor2=m_cPicture2[nSeek];
	switch(nX&3)
	{
	case 0:
		nColor1=m_cPalette[(nColor1&0xc0)>>6];
		nColor2=m_cPalette[(nColor2&0xc0)>>6];
		break;
	case 1:
		nColor1=m_cPalette[(nColor1&0x30)>>4];
		nColor2=m_cPalette[(nColor2&0x30)>>4];
		break;
	case 2:
		nColor1=m_cPalette[(nColor1&0xc)>>2];
		nColor2=m_cPalette[(nColor2&0xc)>>2];
		break;
	case 3:
		nColor1=m_cPalette[nColor1&0x3];
		nColor2=m_cPalette[nColor2&0x3];
		break;
	default:
		throw CImageException(CImageException::IE_BAD_PARAMS);
	}
	nColor1=m_pColorProvider->getColor(nColor1);
	nColor2=m_pColorProvider->getColor(nColor2);
	nResult=mixColors(nColor1, nColor2);
	return nResult;
}

////////////////////////////////////////////////////////

const char* CInt::getFileName(void)
{
	return m_sFileName.c_str();
}

////////////////////////////////////////////////////////

const char* CInt::getFileSize(void)
{
	return "160x200; 7 colours";
}

////////////////////////////////////////////////////////

const char* CInt::getFileProperties(void)
{
	return "No properties supported.";
}

////////////////////////////////////////////////////////

unsigned int CInt::mixColors(unsigned int _nColor1, unsigned int _nColor2)
{
	unsigned int nRed1, nRed2;
	unsigned int nGreen1, nGreen2;
	unsigned int nBlue1, nBlue2;
	unsigned int nResult;

	if(_nColor1==_nColor2)
		return _nColor1;
	nRed1=RED(_nColor1);
	nGreen1=GREEN(_nColor1);
	nBlue1=BLUE(_nColor1);
	nRed2=RED(_nColor2);
	nGreen2=GREEN(_nColor2);
	nBlue2=BLUE(_nColor2);
	nResult=((nRed1+nRed2)>>1)<<16;
	nResult+=((nGreen1+nGreen2)>>1)<<8;
	nResult+=(nBlue1+nBlue2)>>1;
	return nResult;
}

////////////////////////////////////////////////////////
