/* Yvonne 1.0
 * by Rafal BOBER Ciepiela
 * contact: bob_er@users.sourceforge.net
 * 2006
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "mic.h"

using namespace std;

////////////////////////////////////////////////////////

CMic::CMic(CColorProvider* _pColors):CImage(_pColors)
{
	m_cPicture=NULL;
	return;
}

////////////////////////////////////////////////////////

CMic::~CMic(void)
{
	if(m_cPicture!=NULL)
		delete m_cPicture;
	return;
}

////////////////////////////////////////////////////////

bool CMic::open(char* _szPath)
{
	if(_szPath==NULL)
		throw CImageException(CImageException::IE_BAD_PARAMS);
	m_sFileName=_szPath;
	m_fInFile.open(_szPath, ios::in);
	if(m_fInFile.is_open()==false)
		throw CImageException(CImageException::IE_CANNOT_OPEN_FILE);
	m_cPicture=new unsigned char[IMAGE_SIZE_FULLSCREEN];
	m_fInFile.read(reinterpret_cast<char *>(m_cPicture), IMAGE_SIZE_FULLSCREEN);
	m_fInFile.read(reinterpret_cast<char *>(m_cPalette), 4);
	if(m_fInFile.eof()==true)
		throw CImageException(CImageException::IE_CANNOT_READ_FILE);
	m_fInFile.close();
	m_nWidth=320;
	m_nHeight=192;
	return true;
}

////////////////////////////////////////////////////////

unsigned int CMic::getPixel(unsigned int _nX, unsigned int _nY)
{
	unsigned int nX;
	unsigned int nSeek;
	unsigned int nResult;

	if((_nX>=m_nWidth) || (_nY>=m_nHeight))
		throw CImageException(CImageException::IE_BAD_PARAMS);
	nX=_nX>>1;
	nSeek=(40*_nY)+(nX>>2);
	nResult=m_cPicture[nSeek];
	switch(nX&3)
	{
	case 0:
		nResult=m_cPalette[(nResult&0xc0)>>6];
		break;
	case 1:
		nResult=m_cPalette[(nResult&0x30)>>4];
		break;
	case 2:
		nResult=m_cPalette[(nResult&0xc)>>2];
		break;
	case 3:
		nResult=m_cPalette[nResult&0x3];
		break;
	default:
		throw CImageException(CImageException::IE_BAD_PARAMS);
	}
	nResult=m_pColorProvider->getColor(nResult);
	return nResult;
}

////////////////////////////////////////////////////////

const char* CMic::getFileName(void)
{
	return m_sFileName.c_str();
}

////////////////////////////////////////////////////////

const char* CMic::getFileSize(void)
{
	return "160x192; 4 colours";
}

////////////////////////////////////////////////////////

const char* CMic::getFileProperties(void)
{
	return "No properties supported.";
}

////////////////////////////////////////////////////////
