/* Yvonne 1.0
 * by Rafal BOBER Ciepiela
 * contact: bob_er@users.sourceforge.net
 * 2006
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdlib.h>
#include <strings.h>

#include "rc_file.h"

using namespace std;

const char CRCFile::m_vRCFile[]=".yvonnerc";

////////////////////////////////////////////////////////

CRCFile::CRCFile(void)
{
	m_sFileName=getenv("HOME");
	m_sFileName+="/";
	m_sFileName+=m_vRCFile;
	setDefault();
	return;
}

////////////////////////////////////////////////////////

bool CRCFile::open(void)
{
	fstream iFile;
	int a;

	iFile.open(m_sFileName.c_str(), ios::in);
	if(iFile.is_open()==false)
	{
		iFile.close();
		save();
		return false;
	}
	while(iFile.getline(m_szLine, LINE_SIZE))
	{
		if((m_szLine[0]==';') || (m_szLine[0]=='#'))
			continue;
		if(strncasecmp("directory", m_szLine, 9)==0)
		{
			a=9;
			if(skipSpaces(&a)==true)
				m_sDefaultDirectory=&m_szLine[a];
			else
				throw CRCFileException(CRCFileException::RCF_PARSE_ERROR);
		}
		else if(strncasecmp("palette", m_szLine, 7)==0)
		{
			a=7;
			if(skipSpaces(&a, false)==true)
				m_sDefaultPalette=&m_szLine[a];
			else
				throw CRCFileException(CRCFileException::RCF_PARSE_ERROR);
		}
		else if(strncasecmp("pixel", m_szLine, 5)==0)
		{
			a=5;
			if(skipSpaces(&a)==true)
			{
				m_nPixelSize=atoi(&m_szLine[a]);
				if((m_nPixelSize<1) || (m_nPixelSize>16))
					throw CRCFileException(CRCFileException::RCF_PARSE_ERROR);
			}
			else
				throw CRCFileException(CRCFileException::RCF_PARSE_ERROR);
		}
		else
			throw CRCFileException(CRCFileException::RCF_PARSE_ERROR);
	}
	iFile.close();
	return true;
}


////////////////////////////////////////////////////////

bool CRCFile::skipSpaces(int *_pnSeek, bool _bErrorOnEOL)
{
	int a;

	if(_pnSeek==NULL)
		throw CRCFileException(CRCFileException::RCF_BAD_PARAMS);
	a=*_pnSeek;
	while((m_szLine[a]==' ') || (m_szLine[a]=='\t'))
		++a;
	if((_bErrorOnEOL==true) && ((a==0) || (m_szLine[a]=='\n') || (m_szLine[a]=='\0')))
		return false;
	*_pnSeek=a;
	return true;
}

////////////////////////////////////////////////////////

void CRCFile::setDefault(void)
{
	m_nPixelSize=1;
	m_sDefaultDirectory=getenv("HOME");
	m_sDefaultPalette.clear();
	return;
}

////////////////////////////////////////////////////////

void CRCFile::setDefaultDirectory(char* _szPath)
{
	if(_szPath==NULL)
		throw CRCFileException(CRCFileException::RCF_BAD_PARAMS);
	else
		m_sDefaultDirectory=_szPath;
	return;
}

////////////////////////////////////////////////////////

void CRCFile::setDefaultPalette(char* _szPath)
{
	if(_szPath==NULL)
		throw CRCFileException(CRCFileException::RCF_BAD_PARAMS);
	else
		m_sDefaultPalette=_szPath;
	return;
}

////////////////////////////////////////////////////////

void CRCFile::setPixelSize(int _nPixel)
{
	if((_nPixel<1) || (_nPixel>16))
		throw CRCFileException(CRCFileException::RCF_BAD_PARAMS);
	else
		m_nPixelSize=_nPixel;
	return;
}

////////////////////////////////////////////////////////

bool CRCFile::save(void)
{
	fstream oFile;

	oFile.open(m_sFileName.c_str(), ios::out|ios::trunc);
	if(oFile.is_open()==false)
		throw CRCFileException(CRCFileException::RCF_CANNOT_OPEN_FILE);
	oFile<<"; Yvonne .rc file"<<endl;
	oFile<<"directory		"<<m_sDefaultDirectory<<endl;
	oFile<<"palette			"<<m_sDefaultPalette<<endl;
	oFile<<"pixel				"<<m_nPixelSize<<endl;
	oFile.close();
	return true;
}

////////////////////////////////////////////////////////
