/* Yvonne 1.0
 * by Rafal BOBER Ciepiela
 * contact: bob_er@users.sourceforge.net
 * 2006
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __YVONNE_RCFILE__
#define __YVONNE_RCFILE__

#include <exception>
#include <fstream>
#include <string>

const int LINE_SIZE=256;

////////////////////////////////////////////////////////

class CRCFileException:public std::exception
{
public:
	enum RCFReason {
		RCF_CANNOT_INITIALIZE=1,
		RCF_CANNOT_OPEN_FILE=2,
		RCF_PARSE_ERROR=3,
		RCF_BAD_PARAMS=4
	};

	CRCFileException(RCFReason _eReason) throw():m_eReason(_eReason) {};
	RCFReason getReason(void) { return m_eReason; }
	char* what(void)
	{
		switch(m_eReason)
		{
		case RCF_CANNOT_INITIALIZE:
			return "Cannot initialize.";
		case RCF_CANNOT_OPEN_FILE:
			return "Cannot open .rc file.";
		case RCF_PARSE_ERROR:
			return "Parse error.";
		case RCF_BAD_PARAMS:
			return "Bad parameters.";
		}
		return "Unknown exception.";
	}

protected:
	RCFReason m_eReason;

};

////////////////////////////////////////////////////////

class CRCFile
{
public:
	CRCFile(void);
	bool open(void);
	const int getPixelSize(void)
		{ return m_nPixelSize; }
	const char* getDefaultDirectory(void)
		{ return m_sDefaultDirectory.c_str(); }
	const char* getDefaultPalette(void)
		{ return m_sDefaultPalette.c_str(); }
	void setDefault(void);
	void setDefaultDirectory(char* _szPath);
	void setDefaultPalette(char*);
	void setPixelSize(int _nPixel);
	bool save(void);

private:
	std::string m_sFileName;
	std::string m_sDefaultDirectory;
	std::string m_sDefaultPalette;
	int m_nPixelSize;
	static const char m_vRCFile[];
	char m_szLine[LINE_SIZE];

	bool skipSpaces(int *, bool _bErrorOnEOL=true);	
};

////////////////////////////////////////////////////////

#endif /* __YVONNE_RCFILE__ */
