/* Yvonne 1.0
 * by Rafal BOBER Ciepiela
 * contact: bob_er@users.sourceforge.net
 * 2006
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "widget.h"
#include "widget.moc"

////////////////////////////////////////////////////////

CWidget::CWidget(QMainWindow* _pParent)
{
	m_pModel=NULL;
	m_pImage=NULL;
	m_pGraphics=NULL;
	m_pPixmapItem=NULL;
	m_pParent=_pParent;
	m_tRCFile.open();
	m_nPixelSize=m_tRCFile.getPixelSize();
	m_sDefaultDirectory=m_tRCFile.getDefaultDirectory();
	m_sDefaultPalette=m_tRCFile.getDefaultPalette();
	m_tColors.open(m_sDefaultPalette.toAscii().data());
	m_tUi.setupUi(m_pParent);
	return;
}

////////////////////////////////////////////////////////

CWidget::~CWidget(void)
{
	DEL_POINTER(m_pModel)
	DEL_POINTER(m_pImage)
	DEL_POINTER(m_pGraphics)
	DEL_POINTER(m_pPixmapItem)
	return;
}

////////////////////////////////////////////////////////

void CWidget::init(char* _szPath)
{
	QString sPath;

	m_pModel=new QDirModel();
	m_pModel->setFilter(QDir::Files|QDir::Hidden|QDir::AllDirs);
	m_tUi.fs_browser->setModel(m_pModel);
	prepareScene();
	if(_szPath!=NULL)
	{
		sPath=_szPath;
		createImage(sPath);
	}
	else
		sPath=m_sDefaultDirectory;
	for(int a=0;a<sPath.length();a++)
	{
		if(sPath[a]=='/')
			m_tUi.fs_browser->expand(m_pModel->index(sPath.left(a)));
	}
	m_tUi.fs_browser->expand(m_pModel->index(sPath));
	m_tUi.fs_browser->setRootIndex(m_pModel->index(QDir::rootPath()));
	m_tUi.fs_browser->resizeColumnToContents(0);

	connect(m_tUi.fs_browser, SIGNAL(activated(const QModelIndex&)), this, SLOT(onActivate(const QModelIndex&)));
	connect(m_tUi.actionExit, SIGNAL(activated()), m_pParent, SLOT(close()));
	connect(m_tUi.actionSave_As, SIGNAL(activated()), this, SLOT(onSaveAs()));
	connect(m_tUi.actionSettings, SIGNAL(activated()), this, SLOT(onSettings()));
	connect(m_tUi.actionLoad_Default_Settings, SIGNAL(activated()), this, SLOT(onLoadDefault()));
	connect(m_tUi.actionSave_Settings, SIGNAL(activated()), this, SLOT(onSaveSettings()));
	connect(m_tUi.actionAbout, SIGNAL(activated()), this, SLOT(onAbout()));

	m_tUi.actionExit->setShortcut(tr("Ctrl+X"));
	m_tUi.actionSave_As->setShortcut(tr("Ctrl+S"));
	m_tUi.actionSettings->setShortcut(tr("Ctrl+T"));
	m_tUi.actionLoad_Default_Settings->setShortcut(tr("Ctrl+D"));
	m_tUi.actionSave_Settings->setShortcut(tr("Ctrl+A"));
	m_tUi.actionAbout->setShortcut(tr("Ctrl+H"));
	return;
}

////////////////////////////////////////////////////////

void CWidget::onActivate(const QModelIndex& _index)
{
	QString sPath;

	if(m_pModel->isDir(_index)==true)
		return;
	sPath=m_pModel->filePath(_index);
	createImage(sPath);
	return;
}

////////////////////////////////////////////////////////

void CWidget::createImage(QString& _sPath)
{
	try
	{
		checkPath(_sPath);
		m_pImage->open(_sPath.toAscii().data());
		display();
	}
	catch(CImageException &e)
	{
		QMessageBox::critical(m_pParent, NAME, e.what());
		delete m_pImage;
		m_pImage=NULL;
	}
	return;
}

////////////////////////////////////////////////////////

bool CWidget::checkPath(QString& _sPath)
{
	QString sPath=_sPath.toLower();

	if(m_pImage!=NULL)
		delete m_pImage;
	if((sPath.endsWith(".ap3")==true) || (sPath.endsWith(".apv")==true) || (sPath.endsWith(".cin")==true))
		m_pImage=new CAp3(&m_tColors);
	else if((sPath.endsWith(".apc")==true) || (sPath.endsWith(".plm")==true))
		m_pImage=new CApc(&m_tColors);
	else if(sPath.endsWith(".gr8")==true)
		m_pImage=new CGr8(&m_tColors);
	else if((sPath.endsWith(".g09")==true) || (sPath.endsWith(".bg9")==true))
		m_pImage=new CG09(&m_tColors);
	else if(sPath.endsWith(".gr9")==true)
		m_pImage=new CGr9(&m_tColors);
	else if(sPath.endsWith(".hpm")==true)
		m_pImage=new CHpm(&m_tColors);
	else if((sPath.endsWith(".int")==true) || (sPath.endsWith(".inp")==true))
		m_pImage=new CInt(&m_tColors);
	else if(sPath.endsWith(".mic")==true)
		m_pImage=new CMic(&m_tColors);
	else
		m_pImage=new CImage(&m_tColors);
	return true;
}

////////////////////////////////////////////////////////

void CWidget::prepareScene(void)
{
	DEL_POINTER(m_pGraphics)
	m_pGraphics=new QGraphicsScene(0, 0, IMAGE_WIDTH_MAX*MAX_PIXELSIZE, IMAGE_HEIGHT_MAX*MAX_PIXELSIZE, m_tUi.graphics);
	m_pGraphics->setBackgroundBrush(Qt::SolidPattern);
	m_tUi.graphics->setScene(m_pGraphics);
	m_tUi.graphics->show();
	return;
}

////////////////////////////////////////////////////////

bool CWidget::display(void)
{
	QImage* pDisplayImage;
	int nSeekX;
	int nSeekY;
	int nWidth;
	int nHeight;
	unsigned int nPixel;

	if(m_pImage==NULL)
		return false;
	nWidth=m_pImage->getWidth();
	nHeight=m_pImage->getHeight();
	pDisplayImage=new QImage(IMAGE_WIDTH_MAX, IMAGE_HEIGHT_MAX, QImage::Format_RGB32);
	nSeekX=(IMAGE_WIDTH_MAX-nWidth)>>1;
	nSeekY=(IMAGE_HEIGHT_MAX-nHeight)>>1;
	for(int y=0;y<nHeight;y++)
	{
		for(int x=0;x<nWidth;x++)
		{
			nPixel=m_pImage->getPixel(x, y);
			pDisplayImage->setPixel(nSeekX+x, nSeekY+y, nPixel);
		}
	}
	if(m_pPixmapItem!=NULL)
	{
		m_pGraphics->removeItem(m_pPixmapItem);
		delete m_pPixmapItem;
	}
	m_tPixmap=QPixmap::fromImage(*pDisplayImage);
	m_pPixmapItem=m_pGraphics->addPixmap(m_tPixmap.scaled(IMAGE_WIDTH_MAX*m_nPixelSize, IMAGE_HEIGHT_MAX*m_nPixelSize, Qt::KeepAspectRatio));
	nSeekX=((IMAGE_WIDTH_MAX*MAX_PIXELSIZE)-(nWidth*m_nPixelSize))>>1;
	nSeekY=((IMAGE_HEIGHT_MAX*MAX_PIXELSIZE)-(nHeight*m_nPixelSize))>>1;
	m_pPixmapItem->setPos(nSeekX, nSeekY);
	m_tUi.graphics->centerOn(m_pPixmapItem);
	m_tUi.label_filename->setText(m_pImage->getFileName());
	m_tUi.label_size->setText(m_pImage->getFileSize());
	m_tUi.label_properties->setText(m_pImage->getFileProperties());
	delete pDisplayImage;
	return true;
}

////////////////////////////////////////////////////////

void CWidget::onSaveAs(void)
{
	QDialog* pDialog;

	if(m_pImage==NULL)
	{
		QMessageBox::critical(m_pParent, NAME, "No loaded image.");
		return;
	}
	pDialog=new QDialog(m_pParent);
	m_tSaveAs.setupUi(pDialog);
	m_tSaveAs.combo_fileformat->addItem("BMP", BMP);
	m_tSaveAs.combo_fileformat->addItem("JPG", JPG);
	m_tSaveAs.combo_fileformat->addItem("PNG", PNG);
	m_tSaveAs.combo_fileformat->addItem("XPM", XPM);
	m_tSaveAs.combo_fileformat->setCurrentIndex(2);
	connect(m_tSaveAs.button_browse, SIGNAL(clicked()), this, SLOT(onSaveAsBrowse()));
	pDialog->exec();
	if(pDialog->result()==QDialog::Accepted)
	{
		QString sFileName;
		QPixmap tPixmap;
		int nPosX;
		int nPosY;

		nPosX=(IMAGE_WIDTH_MAX-m_pImage->getWidth())>>1;
		nPosY=(IMAGE_HEIGHT_MAX-m_pImage->getHeight())>>1;
		tPixmap=m_tPixmap.copy(nPosX, nPosY, m_pImage->getWidth(), m_pImage->getHeight());
		tPixmap=tPixmap.scaled(m_pImage->getWidth()*m_nPixelSize, m_pImage->getHeight()*m_nPixelSize, Qt::KeepAspectRatio);
		sFileName=m_tSaveAs.line_filename->text();
		if(sFileName.length()>0)
		{
			if(sFileName.lastIndexOf('.')!=sFileName.length()-4)
			{
				sFileName+=".";
				sFileName+=m_tSaveAs.combo_fileformat->currentText().toLower();
			}
			if(tPixmap.save(sFileName, m_tSaveAs.combo_fileformat->currentText().toAscii().data())==false)
				QMessageBox::critical(m_pParent, NAME, "Cannot write file: '"+sFileName+"'.");
		}
		else
			QMessageBox::critical(m_pParent, NAME, "No file name specified.");
	}
	delete pDialog;
	return;
}

////////////////////////////////////////////////////////

void CWidget::onSaveAsBrowse(void)
{
	m_sSaveAsName=QFileDialog::getSaveFileName(m_pParent, "Save As...", "/tmp", "Images (*.bmp *.jpg *.png *.xpm)");
	m_tSaveAs.line_filename->selectAll();
	m_tSaveAs.line_filename->insert(m_sSaveAsName);
	return;
}

////////////////////////////////////////////////////////

void CWidget::onSettings(void)
{
	QDialog* pDialog;

	pDialog=new QDialog(m_pParent);
	m_tSettings.setupUi(pDialog);
	m_tSettings.line_defaultdir->insert(m_sDefaultDirectory);
	m_tSettings.line_palette->insert(m_sDefaultPalette);
	m_tSettings.spin_pixelsize->setValue(m_nPixelSize);
	connect(m_tSettings.button_browsedir, SIGNAL(clicked()), this, SLOT(onSettingsDirectory()));
	connect(m_tSettings.button_browsepal, SIGNAL(clicked()), this, SLOT(onSettingsPalette()));
	pDialog->exec();
	if(pDialog->result()==QDialog::Accepted)
	{
		m_nPixelSize=m_tSettings.spin_pixelsize->value();
		m_sDefaultDirectory=m_tSettings.line_defaultdir->text();
		m_sDefaultPalette=m_tSettings.line_palette->text();
		updateSettings();
	}
	delete pDialog;
	return;
}

////////////////////////////////////////////////////////

void CWidget::onSettingsDirectory(void)
{
	m_sDefaultDirectory=QFileDialog::getExistingDirectory(m_pParent, "Default Directory", m_sDefaultDirectory, QFileDialog::ShowDirsOnly);
	if(m_sDefaultDirectory.length()>0)
	{
		m_tSettings.line_defaultdir->selectAll();
		m_tSettings.line_defaultdir->insert(m_sDefaultDirectory);
	}
	else
		m_sDefaultDirectory=m_tRCFile.getDefaultDirectory();
	return;
}

////////////////////////////////////////////////////////

void CWidget::onSettingsPalette(void)
{
	m_sDefaultPalette=QFileDialog::getOpenFileName(m_pParent, "Default Palette", m_sDefaultDirectory, "Palettes (*.pal)");
	if(m_sDefaultPalette.length()>0)
	{
		m_tSettings.line_palette->selectAll();
		m_tSettings.line_palette->insert(m_sDefaultPalette);
	}
	else
		m_sDefaultPalette=m_tRCFile.getDefaultPalette();
	return;
}

////////////////////////////////////////////////////////

void CWidget::onLoadDefault(void)
{
	m_tRCFile.setDefault();
	m_sDefaultDirectory=m_tRCFile.getDefaultDirectory();
	m_sDefaultPalette=m_tRCFile.getDefaultPalette();
	m_nPixelSize=m_tRCFile.getPixelSize();
	return;
}

////////////////////////////////////////////////////////

void CWidget::onSaveSettings(void)
{
	m_tRCFile.save();
	return;
}

////////////////////////////////////////////////////////

void CWidget::updateSettings(void)
{
	m_tRCFile.setPixelSize(m_nPixelSize);
	m_tRCFile.setDefaultDirectory(m_sDefaultDirectory.toAscii().data());
	m_tRCFile.setDefaultPalette(m_sDefaultPalette.toAscii().data());
	m_tColors.open(m_sDefaultPalette.toAscii().data());
	return;
}

////////////////////////////////////////////////////////

void CWidget::onAbout(void)
{
	QString sName;

	sName=NAME;
	sName+=" ";
	sName+=VERSION;
	QMessageBox::about(m_pParent, sName, "Atari Picture Viewer.\nby Rafa 'bob_er' Ciepiela, 2006\ncontact: bob_er@users.sourceforge.net\n\nThis is free software.\nCheck 'copying' file for details.");
	return;
}

////////////////////////////////////////////////////////
