/* Yvonne 1.0
 * by Rafal BOBER Ciepiela
 * contact: bob_er@users.sourceforge.net
 * 2006
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __YVONNE_WIDGET__
#define __YVONNE_WIDGET__

#include <QtCore/QModelIndex>
#include <QtCore/QString>
#include <QtGui/QApplication>
#include <QtGui/QDirModel>
#include <QtGui/QFileDialog>
#include <QtGui/QGraphicsPixmapItem>
#include <QtGui/QGraphicsScene>
#include <QtGui/QImage>
#include <QtGui/QMessageBox>
#include <QtGui/QPixmap>

#include "color_provider.h"
#include "image.h"
#include "main_widget.h"
#include "rc_file.h"
#include "saveas_widget.h"
#include "settings_widget.h"
#include "ap3.h"
#include "apc.h"
#include "gr8.h"
#include "g09.h"
#include "gr9.h"
#include "hpm.h"
#include "int.h"
#include "mic.h"

////////////////////////////////////////////////////////

#define DEL_POINTER(x) { if(x!=NULL) delete x; }

const int MAX_PIXELSIZE=16;

////////////////////////////////////////////////////////

class CWidget : public QObject
{
	Q_OBJECT

public:
	CWidget(QMainWindow* window);
	~CWidget(void);
	void init(char* _szPath=NULL);

private:
	enum eOutFileFormat
	{
		BMP=0,
		JPG=1,
		PNG=2,
		XPM=3
	};

	QMainWindow* m_pParent;
	QDirModel* m_pModel;
	Ui::main_widget m_tUi;
	Ui::SaveAs m_tSaveAs;
	Ui::Settings m_tSettings;
	CImage* m_pImage;
	QGraphicsScene* m_pGraphics;
	QGraphicsPixmapItem* m_pPixmapItem;
	QPixmap m_tPixmap;
	QString m_sSaveAsName;
	QString m_sDefaultDirectory;
	QString m_sDefaultPalette;
	CRCFile m_tRCFile;
	CColorProvider m_tColors;
	unsigned int m_nPixelSize;

	bool checkPath(QString& sPath);
	bool display(void);
	void createImage(QString& _sPath);
	void prepareScene(void);
	void updateSettings(void);

private slots:
	void onActivate(const QModelIndex& _index);
	void onSaveAs(void);
	void onSaveAsBrowse(void);
	void onSettings(void);
	void onLoadDefault(void);
	void onSaveSettings(void);
	void onSettingsDirectory(void);
	void onSettingsPalette(void);
	void onAbout(void);

};

////////////////////////////////////////////////////////

#endif /* __YVONNE_WIDGET__ */
